==> doodle-image_sjs1_3-0.23.0-sources.jar: doodle/image/examples/Epicycloid.scala <==
/*
 * Copyright 2015 Creative Scala
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package doodle
package image
package examples

import cats.instances.list.*
import cats.syntax.traverse.*
import doodle.core.*
import doodle.random.*

import scala.math.BigDecimal

object Epicycloid {
  type Epicycloid = List[(Double, Double, Boolean)]

  val size = 200
  def eval(t: Angle, pattern: Epicycloid): Vec = {
    def loop(pattern: Epicycloid): Vec =
      pattern match {
        case Nil => Vec.zero
        case (weight, freq, flipped) :: tail => {
          val angle = (t * freq)
          (if flipped then Vec(weight * angle.sin, weight * angle.cos)
           else Vec(weight * angle.cos, weight * angle.sin)) + loop(tail)
        }
      }

    loop(pattern) * size.toDouble
  }

  val randomEpicycloid: Random[Epicycloid] = {
    val elt: Random[(Double, Double, Boolean)] =
      for {
        w <- Random.double
        f <- Random.int(1, 12)
        p <- Random.discrete((false -> 0.75), (true -> 0.25))
      } yield (w, f.toDouble, p)

    for {
      n <- Random.int(1, 7)
      elts <- (0 until n).toList.map(_ => elt).sequence
    } yield elts
  }

  def render(epicycloid: Epicycloid): Image =
    Image.path(
      ClosedPath(
        PathElement.moveTo(eval(Angle.zero, epicycloid).toPoint) ::
          (BigDecimal(0.0) to 1.0 by 0.001).map { t =>
            val angle = Angle.turns(t.doubleValue)
            PathElement.lineTo(eval(angle, epicycloid).toPoint)
          }.toList
      )
    )

  def image: Image =
    randomEpicycloid.map(render _).run
}

==> doodle-image_sjs1_3-0.30.0-sources.jar: doodle/image/examples/Epicycloid.scala <==
/*
 * Copyright 2015 Creative Scala
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package doodle
package image
package examples

import cats.instances.list.*
import cats.syntax.traverse.*
import doodle.core.*
import doodle.random.{*, given}

import scala.math.BigDecimal

object Epicycloid {
  type Epicycloid = List[(Double, Double, Boolean)]

  val size = 200
  def eval(t: Angle, pattern: Epicycloid): Vec = {
    def loop(pattern: Epicycloid): Vec =
      pattern match {
        case Nil => Vec.zero
        case (weight, freq, flipped) :: tail => {
          val angle = (t * freq)
          (if flipped then Vec(weight * angle.sin, weight * angle.cos)
           else Vec(weight * angle.cos, weight * angle.sin)) + loop(tail)
        }
      }

    loop(pattern) * size.toDouble
  }

  val randomEpicycloid: Random[Epicycloid] = {
    val elt: Random[(Double, Double, Boolean)] =
      for {
        w <- Random.double
        f <- Random.int(1, 12)
        p <- Random.discrete((false -> 0.75), (true -> 0.25))
      } yield (w, f.toDouble, p)

    for {
      n <- Random.int(1, 7)
      elts <- (0 until n).toList.map(_ => elt).sequence
    } yield elts
  }

  def render(epicycloid: Epicycloid): Image =
    Image.path(
      ClosedPath(
        PathElement.moveTo(eval(Angle.zero, epicycloid).toPoint) ::
          (BigDecimal(0.0) to 1.0 by 0.001).map { t =>
            val angle = Angle.turns(t.doubleValue)
            PathElement.lineTo(eval(angle, epicycloid).toPoint)
          }.toList
      )
    )

  def image: Image =
    randomEpicycloid.map(render _).run
}
